"""Metadata for tmuxp package."""

from __future__ import annotations

import logging

logger = logging.getLogger(__name__)

__title__ = "tmuxp"
__package_name__ = "tmuxp"
__version__ = "1.68.0"
__description__ = "tmux session manager"
__email__ = "tony@git-pull.com"
__author__ = "Tony Narlock"
__github__ = "https://github.com/tmux-python/tmuxp"
__docs__ = "https://tmuxp.git-pull.com"
__tracker__ = "https://github.com/tmux-python/tmuxp/issues"
__changes__ = "https://github.com/tmux-python/tmuxp/blob/master/CHANGES"
__pypi__ = "https://pypi.org/project/tmuxp/"
__license__ = "MIT"
__copyright__ = "Copyright 2013- Tony Narlock"
